package chat.presentation;

import chat.ChatApplication;
import chat.business.Discussion;

// Standard imports
import java.io.IOException;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.util.KeywordValueException;

import org.w3c.dom.html.HTMLBodyElement;
import org.w3c.dom.html.HTMLElement;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLFontElement;

public class EntryFormPresentation implements HttpPresentation {

  private static EntryFormHTML entryForm;

  public void run(HttpPresentationComms comms) throws HttpPresentationException, IOException {
    try{
      maybeAddMsg(comms);
      fillInFields(comms);
    }catch(Exception e){
      e.printStackTrace();
    }

  }

  /**
   *    If is not the first request from a user (or a refresh), then
   *  we are being called by the form we emitted last time. In this case
   *  we should process the message being sent in. If it's the clear
   *  command (optional), then clear the message list. Most of the time
   *  we just add the message to the current discussion. Note how the
   *  HTML/HTTP and the logic of keeping messages is kept separate.
   *    If there is no message, do nothing.
   */
  static public void maybeAddMsg(HttpPresentationComms comms) throws Exception {
    ChatApplication myApp = (ChatApplication) comms.application;
    String name = comms.request.getParameter("userName");

    if (name == null)
      return;
    String text = comms.request.getParameter("text");
    if (text == null)
      return;
    String clearCmd = myApp.getClearCommand();
    if ((clearCmd.length() != 0) && text.equals(clearCmd))
      Discussion.clear();
    else
      Discussion.addMessage(name, text);
  }

  /**
   *  Set up the fields.
   */
  static public void fillInFields(HttpPresentationComms comms) throws Exception {
    ChatApplication myApp = (ChatApplication) comms.application;
    String name = comms.request.getParameter("userName");

    // Background color.
    entryForm = (EntryFormHTML)comms.xmlcFactory.create(EntryFormHTML.class);
    HTMLBodyElement color = entryForm.getElementColor();
    color.setBgColor(myApp.getBgColor());
    comms.response.writeDOM(entryForm);

    /*
    *  If it's IE, they won't see the spinning icon while the page loads
    *  forever. So if it's not IE, then add a little explanation.
     */
    String message = "Reload the page if your messages stop appearing.";
    String ua = comms.request.getHeader("User-Agent");
    if ((ua == null) || (ua.indexOf("MSIE") == -1))
      message += "<br>It is normal for the page to " +
      "keep loading indefinitely.";
    message += "<br>This is a demonstration <a href=http://www.enhydra.org " +
               "target=enhydra>Enhydra</a> application.";
    entryForm.setTextMessage(message);
    comms.response.writeDOM(entryForm);

  }

}



