/**
 * Servlet ServletEditor
 * <P>
 * @author: Stefanovic Nenad, Bojanic Sasa
 * @Date:   August 2001 - May 2003
 */

package fops.presentation;

import java.io.*;
import java.net.*;
import java.util.Date;
import java.text.DateFormat;

import org.apache.fop.apps.Driver;
import org.apache.fop.apps.Options;
import org.apache.fop.apps.Version;
import org.apache.fop.apps.XSLTInputHandler;
import org.apache.fop.messaging.MessageHandler;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.avalon.framework.logger.Logger;


// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.*;


public class ServletEditor implements HttpPresentation {

   private Logger log = null;

   private String DateHTML = null;
   private String NameHTML = null;
   private String SubjHTML = null;
   private String AppletHTML = null;
   private String xmlParam =null;
   private String xslParam = null;
   private String foParam = null;

   private File userConfigFile;
   
   boolean noParams = false;

   public void run(HttpPresentationComms comms) throws Exception {
      if (log == null)
      {
         log = new ConsoleLogger(ConsoleLogger.LEVEL_WARN);
         MessageHandler.setScreenLogger(log);
      }

      xmlParam = comms.request.getParameter("xml");
      xslParam = comms.request.getParameter("xsl");
      foParam = comms.request.getParameter("fo");
      DateHTML=comms.request.getParameter("date");
      NameHTML=comms.request.getParameter("name");
      SubjHTML=comms.request.getParameter("subject");
      AppletHTML = comms.request.getParameter("context");

		//User defined configuration file for embeded fonts
      userConfigFile=((fops.Fopes)comms.application).getUserConfigFile();
		Options options = new Options(userConfigFile);
		

      if((xmlParam != null) && (xslParam != null)){
      	//Rendering a PDF from XML+XSL files
         System.out.println("PDF from XML: "+xmlParam+" and XSL: "+xslParam);
         XSLTInputHandler input = new XSLTInputHandler(new File(xmlParam), new File(xslParam));
         renderXML(input, comms.response);

      }else if(foParam !=null){
      	// Rendering a PDF from FO file
         FileInputStream fofile = new FileInputStream(foParam);
         renderFOFile(fofile, comms.response);

		}else if(DateHTML!=null || NameHTML!=null || SubjHTML!=null || AppletHTML!=null){
      	// Rendering a PDF from applet parameters
         renderFO(comms.response);
                  
   	}else{
			System.out.println("There is no any request parameter given!!!");
			noParams = true;
         renderFO(comms.response);
      }
    }

    /**
     * renders a HTML fields and Applet contents inputsource
     * into a PDF file which is rendered directly to the response
     * object's OutputStream
     */
	private void renderFO(HttpPresentationResponse response) throws Exception {
         
		ByteArrayOutputStream out = new ByteArrayOutputStream();

		response.setContentType("application/pdf");
		
		
		StringReader str;
		if(noParams)
			str = defaultText();
		else
			str = inputText();
		Driver driver = new Driver(new InputSource(str), out);
		driver.setLogger(log);
		driver.setRenderer(Driver.RENDER_PDF);

		driver.run();

		byte[] content = out.toByteArray();
		response.setContentLength(content.length);
		response.getOutputStream().write(content);
		response.getOutputStream().flush();

   }

   private void renderXML(XSLTInputHandler input,
                         HttpPresentationResponse response) throws Exception {

		ByteArrayOutputStream out = new ByteArrayOutputStream();

		response.setContentType("application/pdf");
         
		Driver driver = new Driver();
		driver.setLogger(log);
		driver.setRenderer(Driver.RENDER_PDF);
		driver.setOutputStream(out);
		driver.render(input.getParser(), input.getInputSource());

		byte[] content = out.toByteArray();
		response.setContentLength(content.length);
		response.getOutputStream().write(content);
		response.getOutputStream().flush();
		
   }

   /**
    * renders an FO inputsource into a PDF file which is rendered
    * directly to the response object's OutputStream
    */
   private void renderFOFile(FileInputStream foFile,
                         HttpPresentationResponse response) throws Exception {
		ByteArrayOutputStream out = new ByteArrayOutputStream();

		response.setContentType("application/pdf");

		Driver driver = new Driver();
		driver.setLogger(log);
		driver.setRenderer(Driver.RENDER_PDF);
		driver.setInputSource(new InputSource(foFile));
		driver.setOutputStream(out);
		driver.run();

		byte[] content = out.toByteArray();
		response.setContentLength(content.length);
		response.getOutputStream().write(content);
		response.getOutputStream().flush();
		
   }

    /**
     * creates a SAX parser, using the value of org.xml.sax.parser
     * defaulting to org.apache.xerces.parsers.SAXParser
     *
     * @return the created SAX parser
     */
   static XMLReader createParser() throws Exception {
		String parserClassName = System.getProperty("org.xml.sax.parser");

		if (parserClassName == null) {
			parserClassName = "org.apache.xerces.parsers.SAXParser";
		}
		return (XMLReader) Class.forName(parserClassName).newInstance();
   }

    /**
     * Method inputText
     * Creates a XSLFO file with inserted parameters from HTML.
     * Noncomptibility: FOP 0.19 -     <fo:page-sequence master-name="simple">
     *                  FOP 0.20.4 -   <fo:page-sequence master-reference="simple">
     * @return   a StringReader
     *
     */
	private StringReader inputText() {
      String text = new String();
      text += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
      text += "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n";
      text += "  <fo:layout-master-set>\n";
      text += "     <fo:simple-page-master master-name=\"simple\" page-height=\"29.7cm\" page-width=\"21cm\" " + "margin-top=\"1cm\" margin-bottom=\"2cm\" margin-left=\"2.5cm\" margin-right=\"2.5cm\">\n";
      text += "        <fo:region-body margin-top=\"3cm\"/>\n";
      text += "        <fo:region-before extent=\"3cm\"/>\n";
      text += "        <fo:region-after extent=\"1.5cm\"/>\n";
      text += "     </fo:simple-page-master>\n";
      text += "  </fo:layout-master-set>\n";
      text += "  <fo:page-sequence master-reference=\"simple\">\n";
      text += "     <fo:flow flow-name=\"xsl-region-body\">\n";
      text += "        <fo:block font-size=\"18pt\" font-family=\"sans-serif\" line-height=\"24pt\" " + "space-after.optimum=\"15pt\" background-color=\"blue\" color=\"white\" " + "text-align=\"center\" padding-top=\"3pt\">\n";
      text += "           Demo PDF file from applet \n";
      text += "        </fo:block>\n";
      text += "        <fo:table font-family=\"Helvetica\" space-after.optimum=\"20pt\">\n";
      text += "           <fo:table-column column-width=\"20mm\" />\n";
      text += "           <fo:table-column column-width=\"150mm\" />\n";
      text += "           <fo:table-body>\n";
      text += "              <fo:table-row>\n";
      text += "                 <fo:table-cell>\n";
      text += "                       <fo:block color=\"blue\" font-weight=\"bold\">Date:</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block>" + DateHTML + "</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "              </fo:table-row>\n";
      text += "              <fo:table-row>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block color=\"blue\" font-weight=\"bold\">Name:</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block>" + NameHTML + "</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "              </fo:table-row>\n";
      text += "              <fo:table-row>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block color=\"blue\" font-weight=\"bold\">Subject:</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block>" + SubjHTML + "</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "              </fo:table-row>\n";
      text += "           </fo:table-body>\n";
      text += "        </fo:table>\n";
      text += AppletHTML;
      text += "     </fo:flow>\n";
      text += "  </fo:page-sequence>\n";
      text += "</fo:root>\n";

      StringReader foText = new StringReader(text);

      return foText;
   }
   
   private StringReader defaultText() {
      String text = new String();
      text += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
      text += "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n";
      text += "  <fo:layout-master-set>\n";
      text += "     <fo:simple-page-master master-name=\"simple\" page-height=\"29.7cm\" page-width=\"21cm\" " + "margin-top=\"1cm\" margin-bottom=\"2cm\" margin-left=\"2.5cm\" margin-right=\"2.5cm\">\n";
      text += "        <fo:region-body margin-top=\"3cm\"/>\n";
      text += "        <fo:region-before extent=\"3cm\"/>\n";
      text += "        <fo:region-after extent=\"1.5cm\"/>\n";
      text += "     </fo:simple-page-master>\n";
      text += "  </fo:layout-master-set>\n";
      text += "  <fo:page-sequence master-reference=\"simple\">\n";
      text += "     <fo:flow flow-name=\"xsl-region-body\">\n";
      text += "        <fo:block font-size=\"18pt\" font-family=\"sans-serif\" line-height=\"24pt\" " + "space-after.optimum=\"15pt\" background-color=\"blue\" color=\"white\" " + "text-align=\"center\" padding-top=\"3pt\">\n";
      text += "           Demo PDF file \n";
      text += "        </fo:block>\n";
      text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
      text += "           <fo:inline font-family=\"sans-serif\" font-size=\"18pt\" font-weight=\"bold\" >There is no any request parameter given!!!</fo:inline>\n";
      text += "        </fo:block>\n";
      text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
      text += "           <fo:inline font-family=\"sans-serif\" font-size=\"16pt\" ></fo:inline>\n";
      text += "        </fo:block>\n";
      text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
      text += "           <fo:inline font-family=\"sans-serif\" font-size=\"12pt\" >Please, see documentation for details.</fo:inline>\n";
      text += "        </fo:block>\n";
      text += "     </fo:flow>\n";
      text += "  </fo:page-sequence>\n";
      text += "</fo:root>\n";
   	
      StringReader defaultText = new StringReader(text);

      return defaultText;
   }
}

