package projectmanagement.presentation.employees;

import projectmanagement.presentation.*;
import projectmanagement.business.employee.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Shows a list of employees to administer.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Administering extends BasePO {

   /**
    * Superclass method override. Returns 1.
    */
   protected int getRequiredAuthLevel() {
      return 1;
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {

	   AdministeringHTML page = new AdministeringHTML();

      /*String errorMsg = this.getSessionData().getAndClearUserMessage();
      if(null != errorMsg) {
         page.setTextLblErrorText(errorMsg);
      } else {
         page.getElementLblErrorText().getParentNode().removeChild(page.getElementLblErrorText());
      }*/

      // fetches the table element from HTML document
      HTMLTableElement table = page.getElementTblEmployees();

      try {
         Employee[] employees=EmployeeFactory.getAllEmployees();

         if (employees!=null) {
            for (int i=0; i<employees.length; i++) {
               // creating row in HTML table which describes table properties
               HTMLTableRowElement htmlRE=createNewRow(page,employees[i]);
               // appending row at the end of table
               table.appendChild(htmlRE);
            }
         }
      } catch(Exception ex) {
         this.writeDebugMsg("Error populating list of employees: " + ex);
         throw new ProjectManagementPresentationException("Error getting list of employees: ", ex);
      }

      //Remove template selection row from table
      table.removeChild(page.getElementTemplateRow());

      // write out HTML
      return page;
   }

   /**
   * Creates the new row of HTML table element, based on given parameters.
   */
   private HTMLTableRowElement createNewRow(AdministeringHTML administeringHTML,
         Employee employee) {

      String employeeID="";

      try {
         employeeID=employee.getHandle();
         administeringHTML.setTextTxtFirstName(employee.getFirstName());
         administeringHTML.setTextTxtLastName(employee.getLastName());
         administeringHTML.setTextTxtMobilePhone(employee.getMobilePhone());
         administeringHTML.setTextTxtEmail(employee.getEmail());
      } catch (Exception ex) {}

      // image to get details on employee
      HTMLImageElement detailsImg=administeringHTML.getElementImgDetails();
      detailsImg.setName(employeeID);

      // image to modify employee information
      HTMLImageElement modifyImg=administeringHTML.getElementImgModify();
      modifyImg.setName(employeeID);

      // image to delete employee
      HTMLImageElement deleteImg=administeringHTML.getElementImgDelete();
      deleteImg.setName(employeeID);

      HTMLTableRowElement row=(HTMLTableRowElement)administeringHTML.
         getElementTemplateRow().cloneNode(true);

      return row;
   }

}

